@@ -3,14 +3,12 @@ import asyncio
 import json
 import logging
 import sys
-from functools import partial
 from pathlib import Path
 from typing import Any, Dict, List
 
 import annofabapi
 import pandas
-import requests
-from annofabapi.models import JobType, ProjectMemberRole
+from annofabapi.models import ProjectMemberRole
 
 import annofabcli
 from annofabcli import AnnofabApiFacade
@@ -22,61 +20,14 @@ from annofabcli.common.cli import (
     get_wait_options_from_args,
 )
 from annofabcli.common.dataclasses import WaitOptions
+from annofabcli.common.download import DownloadingFile
 from annofabcli.common.enums import FormatArgument
 
 logger = logging.getLogger(__name__)
 
-
 DEFAULT_WAIT_OPTIONS = WaitOptions(interval=60, max_tries=360)
 
 
-class DownloadingLatestFile:
-    """
-    `AbstractCommandLineInterface`を継承したクラスだと、`multiprocessing.Pool`を実行したときに
-    `AttributeError: Can't pickle local object 'ArgumentParser.__init__.<locals>.identity'`というエラーが発生したので、
-    `ArgumentParser`を除いたクラスを作成した。
-    """
-
-    def __init__(self, service: annofabapi.Resource):
-        self.service = service
-
-    async def wait_until_updated_input_data(self, project_id: str, wait_options: WaitOptions):
-        try:
-            self.service.api.post_project_inputs_update(project_id)
-        except requests.HTTPError as e:
-            if e.response.status_code != requests.codes.conflict:
-                raise e
-
-        loop = asyncio.get_event_loop()
-        partial_func = partial(
-            self.service.wrapper.wait_for_completion,
-            project_id,
-            JobType.GEN_INPUTS_LIST,
-            wait_options.interval,
-            wait_options.max_tries,
-        )
-        result = await loop.run_in_executor(None, partial_func)
-        return result
-
-    async def wait_until_updated_task(self, project_id: str, wait_options: WaitOptions):
-        try:
-            self.service.api.post_project_tasks_update(project_id)
-        except requests.HTTPError as e:
-            if e.response.status_code != requests.codes.conflict:
-                raise e
-
-        loop = asyncio.get_event_loop()
-        partial_func = partial(
-            self.service.wrapper.wait_for_completion,
-            project_id,
-            JobType.GEN_TASKS_LIST,
-            wait_options.interval,
-            wait_options.max_tries,
-        )
-        result = await loop.run_in_executor(None, partial_func)
-        return result
-
-
 class ListInputDataMergedTask(AbstractCommandLineInterface):
     def __init__(self, service: annofabapi.Resource, facade: AnnofabApiFacade, args: argparse.Namespace):
         super().__init__(service, facade, args)
@@ -130,21 +81,21 @@ class ListInputDataMergedTask(AbstractCommandLineInterface):
 
         return True
 
-    def download_latest_files(self, project_id: str, output_dir: Path, is_latest: bool, wait_options: WaitOptions):
-        if is_latest:
-            downloading_obj = DownloadingLatestFile(self.service)
-            loop = asyncio.get_event_loop()
-
-            gather = asyncio.gather(
-                downloading_obj.wait_until_updated_input_data(project_id, wait_options),
-                downloading_obj.wait_until_updated_task(project_id, wait_options),
-            )
-            result = loop.run_until_complete(gather)
-            if len([e for e in result if not e]) > 0:
-                raise RuntimeError("タスク全件ファイル、入力データ全件ファイルの更新に失敗しました。")
-
-        self.service.wrapper.download_project_inputs_url(project_id, str(output_dir / "input_data.json"))
-        self.service.wrapper.download_project_tasks_url(project_id, str(output_dir / "task.json"))
+    def download_json_files(self, project_id: str, output_dir: Path, is_latest: bool, wait_options: WaitOptions):
+        loop = asyncio.get_event_loop()
+        downloading_obj = DownloadingFile(self.service)
+        gather = asyncio.gather(
+            downloading_obj.download_input_data_json_with_async(
+                project_id,
+                dest_path=str(output_dir / "input_data.json"),
+                is_latest=is_latest,
+                wait_options=wait_options,
+            ),
+            downloading_obj.download_task_json_with_async(
+                project_id, dest_path=str(output_dir / "task.json"), is_latest=is_latest, wait_options=wait_options
+            ),
+        )
+        loop.run_until_complete(gather)
 
     def main(self):
         args = self.args
@@ -156,7 +107,7 @@ class ListInputDataMergedTask(AbstractCommandLineInterface):
             super().validate_project(project_id, [ProjectMemberRole.OWNER])
             wait_options = get_wait_options_from_args(get_json_from_args(args.wait_options), DEFAULT_WAIT_OPTIONS)
             cache_dir = annofabcli.utils.get_cache_dir()
-            self.download_latest_files(project_id, cache_dir, args.latest, wait_options)
+            self.download_json_files(project_id, cache_dir, args.latest, wait_options)
             task_json_path = cache_dir / "task.json"
             input_data_json_path = cache_dir / "input_data.json"
         else:
